#################################################################################
#                                                                               #
#                       copyright (c) 2003 Ingeniweb SARL                       #
#                                                                               #
#################################################################################

"""
PlonePopoll main class
"""


from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
import string

from Products.CMFCore.CMFCorePermissions import View, ManageProperties, ListFolderContents
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.PlonePopoll import Permissions
from AccessControl import getSecurityManager, ClassSecurityInfo, Unauthorized
import OFS
from OFS import Folder

from global_symbols import *

import sys
import os.path
import types
import time, random, md5, socket

try:
    _v_network = socket.gethostbyname(socket.gethostname())
except:
    _v_network = random.random() * 100000000000000000L


# PlonePopoll definition Plone1
factory_type_information = {
  'id'             : 'PlonePopoll',
  'portal_type'    : 'PlonePopoll',
  'meta_type'      : 'PlonePopoll',
  'description'    : 'A Poll',
  'content_icon'   : 'PlonePopoll.gif',                          # Icon has to be acquired from a skin folder
  'product'        : 'PlonePopoll',
  'factory'        : 'addPlonePopoll',
  'immediate_view' : 'PlonePopoll_edit',
  'filter_content_types' : 0,                                   # This is for folderish types
  'allowed_content_types': (),                                  # If filter_content_types is true, this field can hold allowed meta_types
  'actions': (
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'PlonePopoll_view',
    'permissions'  : (CMFCorePermissions.View, ),
    'category'      : 'object',
    },
    {
    'id'           : 'edit',
    'name'         : 'Edit',
    'action'       : 'portal_form/PlonePopoll_editForm',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'results',
    'name'         : 'Results',
    'action'       : 'portal_form/PlonePopoll_results',
    'permissions'   : (CMFCorePermissions.View, ),
    'category'      : 'object',
    },
    {
    'id'            : 'metadata',
    'name'          : 'Properties',
    'action'        : 'metadata_edit_form',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    ),
  }


# PlonePopoll definition for Plone2
factory_type_information2 = {
  'id'             : 'PlonePopoll',
  'portal_type'    : 'PlonePopoll',
  'meta_type'      : 'PlonePopoll',
  'description'    : 'A Poll',
  'content_icon'   : 'PlonePopoll.gif',                          # Icon has to be acquired from a skin folder
  'product'        : 'PlonePopoll',
  'factory'        : 'addPlonePopoll',
  'immediate_view' : 'PlonePopoll_view2',
  'filter_content_types' : 0,                                   # This is for folderish types
  'allowed_content_types': (),                                  # If filter_content_types is true, this field can hold allowed meta_types
  'actions': (
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'string:${object_url}/PlonePopoll_view2',
    'permissions'  : (CMFCorePermissions.View, ),
    'category'      : 'object',
    },
    {
    'id'           : 'edit',
    'name'         : 'Edit',
    'action'       : 'string:${object_url}/PlonePopoll_editForm2',
    'permissions'  : (CMFCorePermissions.ModifyPortalContent, ),
    'category'     : 'object',
    },
    {
    'id'           : 'results',
    'name'         : 'Results',
    'action'       : 'string:${object_url}/PlonePopoll_results2',
    'permissions'   : (CMFCorePermissions.View, ),
    'category'      : 'object',
    },
    {
    'id'            : 'metadata',
    'name'          : 'Properties',
    'action'        : 'string:${object_url}/metadata_edit_form',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'content_status_history',
    'name'          : 'State',
    'action'        : 'string:${object_url}/content_status_history',
    'condition'     : 'python:object and portal.portal_workflow.getTransitionsFor(object, object.getParentNode())',
    'permissions'   : (CMFCorePermissions.View,),
    'category'      : 'object_tabs',
    'visible'       : 0
    },
    ),
  }

# The usual factory
def addPlonePopoll(self, id, title = '', REQUEST = {}):
    """
    Factory method for a PlonePopoll object
    """
    obj = PlonePopoll(id, title)
    self._setObject(id, obj)

    getattr(self, id)._post_init()
    if REQUEST.has_key('RESPONSE'):
        return REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_main')

class PlonePopoll(PortalContent, DefaultDublinCoreImpl):
    """PlonePopoll class"""

    #                                                                           #
    #                            BASIC CLASS BEHAVIOUR                          #
    #                                                                           #

    __implements__ = ( PortalContent.__implements__
                     , DefaultDublinCoreImpl.__implements__
                     )

    meta_type = factory_type_information['meta_type']                   # Has not to be the same as FTI but it is the most usual case.

    manage_options = PortalContent.manage_options

    text = ""           # Default text
    
    # Standard security settings
    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)            # $$$ Is this clever ? Isn't it better to make the object private ?
                                                                        # This method makes all class properties & methods protected by View by default
    #can allow, disable discussion per object
    _isDiscussable = 1
    
    # Init method
    security.declarePrivate('__init__')
    def __init__(self, id, title=''):
        """__init__(self, id, title='')"""
        PlonePopoll.inheritedAttribute('__init__')(self)
        self.id = id
        self.title = title
        self.uid = self.make_uuid(id)


    security.declarePrivate('manage_afterClone')
    def manage_afterClone(self, item):
        PortalContent.manage_afterClone(self, item)
        self.uid = self.make_uuid(id)

    def make_uuid(self, *args):
        """Generate unique id"""
        t = long(time.time() * 1000)
        r = long(random.random()*100000000000000000L)
        data = str(t)+' '+str(r)+' '+str(_v_network)+' '+str(args)
        data = md5.md5(data).hexdigest()
        return data

    security.declarePrivate('_post_init')
    def _post_init(self):
        """
        _post_init(self) => Post-init method (that is, method that is called AFTER the class has been set into the ZODB)
        """
        self.indexObject()

    security.declareProtected(View, 'Description')
    def Description(self):
        "Returns the description of the poll, which is in fact the question"
        return self.getPollQuestion()

    security.declareProtected(View, 'getVoteId')
    def getVoteId(self):
        if hasattr(self, 'uid'):
            return self.uid
        else:
            return self.getId()

    #############################################################################
    #                                                                           #
    #                             POLL PROPERTIES                               #
    #                                                                           #
    #############################################################################


    _possible_states = ('enabled', 'disabled', )

    _poll_state = 'enabled'
    _check_multi = 1
    _question = ""
    _choices = [
        ]
    _number_of_choices = 1
    _visibility = 'visible'


    #                                                                           #
    #                               State management                            #
    #                                                                           #
        

    security.declareProtected(access_contents_information, "listPollStates")
    def listPollStates(self,):
        """
        listPollStates(self,)
        return list of strings
        """
        return self._possible_states
    

    security.declareProtected(access_contents_information, "getPollState")
    def getPollState(self,):
        """
        getPollState(self,)
        - return poll state (as a string)
        """
        return self._poll_state
    
    security.declareProtected(access_contents_information, "getResultsVisibility")
    def getResultsVisibility(self,):
        """
        getResultsVisibility(self,)
        - return poll results visibility (as a string)
        """
        return self._visibility
    
    
    security.declareProtected(access_contents_information, "isEnabled")
    def isEnabled(self,):
        """
        isEnabled(self,)
        - return 1 if one can vote within this poll
        """
        return self.getPollState() == "enabled"


    security.declareProtected(access_contents_information, "isDisabled")
    def isDisabled(self,):
        """
        isDisabled(self,)
        - return 1 if one can vote within this poll
        """
        return self.getPollState() == "disabled"

    security.declareProtected(access_contents_information, "isVisible")
    def isVisible(self,):
        """
        isVisible(self,)
        - return 1 if one can see the results
        """
        return self.getResultsVisibility() == "visible"


    security.declareProtected(access_contents_information, "isNotVisible")
    def isNotVisible(self,):
        """
        isNotVisible(self,)
        - return 1 if one cannot see the results
        """
        return self.getResultsVisibility() == "not_visible"

    security.declareProtected(access_contents_information, "enable")
    def enable(self,):
        """
        enable(self,)
        - change poll state
        """
        self._poll_state = "enabled"

    security.declareProtected(access_contents_information, "disable")
    def disable(self,):
        """disable(self,)"""
        self._poll_state = "disabled"

    security.declareProtected(access_contents_information, "setVisible")
    def setVisible(self,):
        """
        setVisible(self,)
        - change poll visibility
        """
        self._visibility = "visible"


    security.declareProtected(access_contents_information, "setNotVisible")
    def setNotVisible(self,):
        """
        setNotVisible(self,)
        - change poll visibility
        """
        self._visibility = "not_visible"


    #                                                                           #
    #                           QUESTIONS / CHOICES MGT                         #
    #                                                                           #


    security.declareProtected(access_contents_information, "getPollQuestion")
    def getPollQuestion(self,):
        """
        getPollQuestion(self,)
        - return poll question
        """
        return self._question

    security.declareProtected(access_contents_information, "getPollNumber")
    def getPollNumber(self,):
        """
        getPollNumber(self,)
         - return poll question
         """
        return self._number_of_choices


    security.declareProtected(access_contents_information, "listPollChoices")
    def listPollChoices(self,):
        """
        listPollChoices(self,)
        - return poll choices as a list of ids
        """
        return self._choices


    security.declareProtected(Permissions.PlonePopoll_editPermission, "setPollOptions")
    def setPollOptions(self, question = '', choices = [], choices_number = 1, check_multi = 1):
        """
        setPollOptions(self, question = '', choices = [], choices_number = 1, check_multi = 1)

        - check_multi is a var that allows PlonePopoll, when it is true, to avoid a user to vote several times
          only its last vote is counted.
        """
        self._check_multi = check_multi
        self._question = question
        self._number_of_choices = choices_number
        if choices:
            self._choices = []
            for c in choices:                                           # We ensure that choices is a list
                self._choices.append(c)
            self._choices = self._choices                               # Ensure persistancy



    #####################################################################
    #                                                                   #
    #                   Poll-related methods and attributes             #
    #                                                                   #
    #####################################################################


    security.declareProtected(access_contents_information, "listChoiceIds")
    def listChoiceIds(self,):
        """
        Return choices list.
        $$$ By now, a choice id is the same as its string. May change in the future, that's why
        we've got TWO methods to get choices : getPollChoices (list of dicts) and listChoiceIds (list of strings)
        """
        return self._choices[:]


    security.declareProtected(Permissions.PlonePopoll_votePermission, "vote")
    def vote(self, index, ):
        """
        vote(self, index, )
        - calls the backend to register the vote
        - Include all the necessary code to pseudo-guarantee the unicity of the vote.
        """
        # Check that choice is valid
        if index >= len(self._choices):
            raise ValueError, "Invalid choice"
        if not self.isEnabled():
            raise RuntimeError, "This poll is not active."
        
        # Ensure unicity of the vote
        
        unicity = getToolByName(self, 'portal_popoll').getVoteUnicity(self.getVoteId(), create=1)
        
        Log(LOG_DEBUG, "Unicity", unicity)
        # Call the method in the backend to store the vote
        
        getToolByName(self, 'portal_popoll').getBackend().vote(self.getVoteId(), index, unicity)


    security.declareProtected(Permissions.PlonePopoll_votePermission, "hasVoted")
    def hasVoted(self,):
        """
        hasVoted(self,)
        - Return 1 if the user has already voted.
        This is based on the unicity token
        """
        unicity = getToolByName(self, 'portal_popoll').getVoteUnicity(self.getVoteId(), create = 0)
        if unicity:
            return getToolByName(self, 'portal_popoll').getBackend().hasAlreadyVoted(self.getVoteId(), unicity)
        return None

    security.declareProtected(Permissions.PlonePopoll_votePermission, "getMaxNumber")
    def getMaxNumber(self,):
        return self._number_of_choices;

    #############################################################################
    #                                                                           #
    #                           RESULTS MANAGEMENT                              #
    #                                                                           #
    #############################################################################


    security.declareProtected(access_contents_information, "getResults")
    def getResults(self, sort = 0):
        """
        getResults(self,)
        -> return results as a TUPLE of tuples (id, count, percentage).

        percentage is 0 <= percentage <= 100
        
        The order is the same as for listChoiceIds if sort if false.
        If sort is true, results are sorted by score (best first)
        """
        res = getToolByName(self, 'portal_popoll').getBackend().getResults(self.getVoteId())
        ret = []
        votes_count = 0
        idx = 0
        for id in self.listChoiceIds():
            # If a choice has not been selected, it is not in res!
            if not res.has_key(idx):
                ret.append((id, 0))
            else:
                votes_count += res[idx]['count']
                ret.append((id, res[idx]['count']))
            idx += 1

        # Create a new tuple that includes the percentage as well
        result=[]
        for r in ret:
           try:
              percentage=float(r[1])/float(votes_count)*100.0
           except:
              percentage=0.0
           #result.append((r[0],r[1],int(percentage)))  ## We should not do the int(). Fixed (KA & MR) !
           result.append((r[0], r[1], percentage))  

        if sort:
            result.sort(lambda x,y: cmp(y[1], x[1]))

        Log(LOG_DEBUG, "getResults for the popoll", result)
        return tuple(result)

    security.declareProtected(access_contents_information, "getVotesCount")
    def getVotesCount(self,):
        """
        getVotesCount(self,)
        - return total number of votes for this poll
        """
        ret = 0
        for id, count, percentage in self.getResults():
            ret += count

        return ret

    security.declareProtected(access_contents_information, "getPersonVoteCount")
    def getPersonVoteCount(self, ):
        return getToolByName(self, 'portal_popoll').getBackend().getPersonVoteCount(self.getVoteId())

    security.declareProtected(Permissions.PlonePopoll_editPermission, "clearResults")
    def clearResults(self, ):
        getToolByName(self, 'portal_popoll').getBackend().clearResults(self.getVoteId())
        
    
    security.declareProtected(View, "getUnicity")
    def getUnicity(self, ):
        return getToolByName(self, 'portal_popoll').getVoteUnicity(self.getVoteId(), create=0)
    
    security.declareProtected(Permissions.PlonePopoll_votePermission, "removeUserVote")
    def removeUserVote(self, ):
        unicity = getToolByName(self, 'portal_popoll').getVoteUnicity(self.getVoteId(), create=0)
        getToolByName(self, 'portal_popoll').getBackend().removeUserVote(self.getVoteId(), unicity)
    
    
    #                                                                           #
    #                             CMF CATALOG SUPPORT                           #
    #                                                                           #

    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        "Returns a concatination of all searchable text"
        # Should be overriden by portal objects
        return "%s %s %s" % (self.Title(), self.Description(), self.text)

# Class instanciation
InitializeClass(PlonePopoll)
