##############################################################################
#
# Copyright (c) 2002 Ingeniweb SARL
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

"""PlonePopoll product"""

from Globals import MessageDialog, DTMLFile      # fakes a method from a DTML file
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Acquisition import Implicit
from Globals import Persistent
from AccessControl.Role import RoleManager
from OFS.SimpleItem import Item
from OFS.PropertyManager import PropertyManager
import OFS
from OFS import ObjectManager, SimpleItem
from DateTime import DateTime
from App import ImageFile
import AccessControl.Role, webdav.Collection
import Products
import os
import string
import shutil
import random

##import ZSQLMethod

from global_symbols import *


class PlonePopollBackend(OFS.SimpleItem.SimpleItem):
    """
    PlonePopollBackend => A simple poll management interface
    """
    meta_type='PlonePopollBackend'
    id       ='PlonePopollBackend'
    title    ='A poll results data manager'


    def __init__(self):
        """
        __init__(self) -> initialization method
        """
        pass


    def _post_init(self):
        """
        _post_init(self) => called from manage_add method
        """
        pass


    #############################################################
    #                                                           #
    #                   Backend interface                       #
    #                                                           #
    #############################################################


    def vote(self, choice, unicity_factor, REQUEST = {}, **kw):
        """
        vote(self, choice, unicity_factor, REQUEST = {}, **kw) => place a vote on the acquired poll

        unicity_factor is a parameter that should be unique to each user. It's the way to keep track
        of multiple votes by a same person.
        """
        raise NotImplementedError, "Has to be redefined in subclasses"



    def getResults(self,):
        """
        getResults(self,) => return results as a dictionary:
        {
        id: {count: 0,},
        }
        """
        raise NotImplementedError, "Has to be redefined in subclasses"





#################################################################
#                                                               #
#                       ZODB BACKEND                            #
#                                                               #
#################################################################



class PlonePopollZODBBackend(PlonePopollBackend):
    """
    PlonePopollZODBBackend => A simple ZODB poll backend implementation
    """
    meta_type = "PlonePopollZODBBackend"


    def __init__(self,):
        """ _results dictionary is made this way:
         {poll_id: { unicity_factor: vote,
                     unicity_factor: vote,
                       ...
                   },
          poll_id: { ...
                   },
          }
        """
        self._results = {}

    def removeUserVote(self, poll_id, unicity_factor):
        """
        remove the previous vote
        """
        self._results[poll_id][unicity_factor] = []
        Log(LOG_DEBUG, "removed user vote", self._results)
        Log(LOG_DEBUG, "User vote :", self._results[poll_id][unicity_factor])
        
    
    def vote(self, poll_id, choice, unicity_factor, REQUEST = {}, **kw):
        """
        vote
        """
        # Record the vote in the _results dictionary
        if not self._results.has_key(poll_id):
            self._results[poll_id] = {}
            
        if not self._results.has_key(poll_id) or not self._results[poll_id].has_key(unicity_factor) :
            self._results[poll_id][unicity_factor] = []

        self._results[poll_id][unicity_factor].append(choice)
        
        self._results = self._results
        Log(LOG_DEBUG, "vote", self._results[poll_id])
        Log(LOG_DEBUG, "vote", self._results[poll_id][unicity_factor])
        Log(LOG_DEBUG, "vote", self._results)


    def hasAlreadyVoted(self, poll_id, unicity_factor):
        """
        hasAlreadyVoted(self, unicity_factor) => return 1 if unicity_factor is already in the database
        """
        r = self._getResults()
        return r.get(poll_id, {}).has_key(unicity_factor)

    def _getResults(self,):
        """
        _getResults(self,) => private method for returning results
        """
        return self._results.copy()


    def getResults(self, poll_id):
        """
        getResults(self,) -> dict {choice: {'count': count}}
        """
        ret = {}
        poll_results = self._results.get(poll_id, {}).items()
        for user, choices in poll_results :
            for choice in choices:
                if not ret.has_key(choice):
                    ret[choice] = {'count': 1}
                else:
                    ret[choice]['count'] += 1
        return ret
            

    def clearResults(self, poll_id):
        """
            clearResults(self,) -> reset the _results dict of the poll_id  to {}
        """
        self._results[poll_id] = {}
        self._p_changed = 1
    
    
    def getPersonVoteCount(self, poll_id):
        """
        getPersonVoteCount(self, poll_id) -> int person count 
        """
        ret = 0
        if not self._results.has_key(poll_id):
            return ret
        else:
            return len(self._results[poll_id])
        
        

class PlonePopollZSQLBackend(OFS.ObjectManager.ObjectManager, OFS.SimpleItem.Item, PlonePopollBackend, ):
    """
    PlonePopollZSQLBackend => A simple ZSQL poll backend implementation
    """
    _results = {}
    meta_type = "PlonePopollZSQLBackend"

    # Management interfaces
    manage_options=(
        OFS.ObjectManager.ObjectManager.manage_options
        + Item.manage_options
        + RoleManager.manage_options
        )


    def _post_init(self,):
        """
        _post_init(self,) => acquisition-aware init method
        """
        self.manage_addProduct['ZSQLMethods'].manage_addZSQLMethod(
            "SQLinit", "SQL table init method", "ZSQLDatabaseAdapter", "",
            """
            CREATE TABLE TM_POLL(
                POLL_Id VARCHAR(255),
                UNICITY_FACTOR VARCHAR(255),
                CHOICE_Id VARCHAR(255)
            )
            """)
        self.manage_addProduct['ZSQLMethods'].manage_addZSQLMethod(
            "SQLvote", "Vote method", "ZSQLDatabaseAdapter", "poll_id choice unicity_factor",
            """
            UPDATE TM_POLL SET CHOICE_Id = <dtml-sqlvar choice type="string">
                           WHERE
                               POLL_Id = <dtml-sqlvar poll_id type="string">
                             AND
                               UNICITY_FACTOR = <dtml-sqlvar unicity_factor type="string">
            """)
        self.manage_addProduct['ZSQLMethods'].manage_addZSQLMethod("SQLgetResults", "return results as an unsorted list. ", "ZSQLDatabaseAdapter", "poll_id", """
        # Put your SQL code here.
        # You should return records made of UNICITY_FACTOR and CHOICE_ID fields. One record per actual (and individual) vote.
        SELECT * FROM TM_POLL WHERE POLL_Id = <dtml-sqlvar poll_id type="string">
        """)
    
    
    def vote(self, poll_id, choice, unicity_factor, REQUEST = {}, **kw):
        """
        vote
        """
        return self.SQLvote(poll_id = poll_id, choice = choice, unicity_factor = unicity_factor)
    

    def getResults(self, poll_id):
        """
        getResults(self,) -> dict {choice: {'count': count}}
        """
        ret = {}
        for user, choices in self._results.get(poll_id, {}).items():
            for choice in choices:
                if not ret.has_key(choice):
                    ret[choice] = {'count': 1}
                else:
                    ret[choice]['count'] += 1
                    
        return ret
        



# THIS IS TEMPORARY ! HAS TO BE REPLACED WITH AN INTERFACE THAT ALLOWS ONE TO CHOOSE HIS BACKEND $$$
def manage_addPlonePopollZODBBackend(self, REQUEST={}, **ignored):
    """ """
    f = PlonePopollZODBBackend()
    try:    self._setObject('PlonePopollBackend', f)
    except: return MessageDialog(
                   title  ='Item Exists',
                   message='This object already contains a PlonePopollBackend',
                   action ='%s/manage_main' % REQUEST['URL1'])
    self.PlonePopollBackend._post_init()
    if REQUEST.has_key('RESPONSE'):
        REQUEST['RESPONSE'].redirect(self.absolute_url() + '/manage_main')


def manage_addPlonePopollZSQLBackend(self, REQUEST={}, **ignored):
    """ """
    f = PlonePopollZSQLBackend()
    try:    self._setObject('PlonePopollBackend', f)
    except: return MessageDialog(
                   title  ='Item Exists',
                   message='This object already contains a PlonePopollBackend',
                   action ='%s/manage_main' % REQUEST['URL1'])
    self.PlonePopollBackend._post_init()
    if REQUEST.has_key('RESPONSE'):
        REQUEST['RESPONSE'].redirect(self.absolute_url() + '/manage_main')



InitializeClass(PlonePopollZODBBackend) 
InitializeClass(PlonePopollZSQLBackend) 

