#
# PlonePopoll tests
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.CMFPlone.tests import PloneTestCase
from Products.PlonePopoll.global_symbols import *
from Acquisition import aq_base
from AccessControl.SecurityManagement import newSecurityManager, noSecurityManager

from AccessControl import getSecurityManager, Unauthorized

from Products.CMFCore.utils import getToolByName

from Products.PloneTestCase import PloneTestCase
PloneTestCase.installProduct('PlonePopoll')

PloneTestCase.setupPloneSite()



try:
    import Log

    Log.LOG_LEVEL = Log.LOG_DEBUG

    Log.LOG_PROCESSOR = {
        Log.LOG_NONE: Log.logFile,
        Log.LOG_CRITICAL: Log.logFile,
        Log.LOG_ERROR: Log.logFile,
        Log.LOG_WARNING: Log.logFile,
        Log.LOG_NOTICE: Log.logFile,
        Log.LOG_DEBUG: Log.logFile,
        }
    from Log import *
    Log(LOG_NOTICE, "Starting %s at %d debug level" % (os.path.dirname(__file__), LOG_LEVEL, ))
except:
    print "Log module not available"
    LOG_DEBUG = None
    LOG_NOTICE = None
    LOG_WARNING = None
    LOG_ERROR = None
    LOG_CRITICAL = None
    def Log(*args, **kw):
        pass
    raise


def sortTuple(t):
    l = list(t)
    l.sort()
    return tuple(l)

class TestPlonePopoll(PloneTestCase.PloneTestCase):

    #utility method
        
    def createMember(self, id, pw, email, roles=('Member',)):
        pr = self.portal.portal_registration
        member = pr.addMember(id, pw, roles, properties={ 'username': id, 'email' : email })
        return member
    
    def createGroup(self, id):
        pg = self.portal.portal_groups
        pg.addGroup(id, '', (), ())
        return pg.getGroupById(id)
    
    def installPlonePopoll(self):
        #self.loginAsPortalOwner()
        self.qi = self.portal.portal_quickinstaller
        self.qi.installProduct('PlonePopoll')
    
    def setupSecurityContext(self,):
        self.logout()
        self.loginAsPortalOwner()
        
        
        #create a bunch of users
        self.user1 = self.createMember('user1', 'abcd4', 'abc@domain.tld')
        self.user2 = self.createMember('user2', 'abcd4', 'abc@domain.tld')
        self.user3 = self.createMember('user3', 'abcd4', 'abc@domain.tld')
        
        self.failUnless('user1' in self.portal.Members.objectIds())
        self.failUnless('user2' in self.portal.Members.objectIds())
        self.failUnless('user3' in self.portal.Members.objectIds())

        #just in case ...
        self.logout()
        self.login('user1')
        self.folder = self.portal.portal_membership.getHomeFolder('user1')
        

    def afterSetUp(self, ):
        #some usefull properties
        self.catalog = self.portal.portal_catalog
        self.workflow = self.portal.portal_workflow
        self.membership = self.portal.portal_membership
        
        #install product
        self.installPlonePopoll()
        
        #setup users and groups
        self.setupSecurityContext()
       
       
    def createPopoll(self, id="sample_poll", folder= None):
        # Login
        self.login("user1")
        if not folder:
            folder = self.membership.getHomeFolder()

        # Content creation
        content_id = id
        folder.invokeFactory(
            type_name = 'PlonePopoll',
            id = content_id
        )
        self.failUnless(content_id in folder.objectIds(), "Object has not been created or not with the right id")
        content = getattr(folder, content_id)
        
        return content
        
    def changePollOptions(self, poll, question='', choices=[], choices_number=1, check_multi=1):
        poll.setPollOptions(
                question = question,
                choices = choices,
                choices_number = choices_number,
                check_multi = check_multi,
        )
        self.assertEqual(poll.getPollNumber(), choices_number)
        self.assertEqual(poll.listPollChoices(), choices)
        self.assertEqual(poll.getPollQuestion(), question)
        self.assertEqual(poll._check_multi, check_multi)
    
    def editPopoll(self, poll, _title="", _question="", _answers=[], _poll_status='enabled', _number=1, _results_visibility='visible'):
        """
        Edit poll with default params
        """
        poll.edit(
            title=_title, 
            )
        self.assertEqual(poll.Title, title)
        self.assertEqual(poll.getPollQuestion(), question)
        self.assertEqual(poll.getPollState(), poll_status)
        self.assertEqual(poll.getPollNumber(), number)
        self.assertEqual(poll.getResultsVisibility(), results_visibility)
        
    
    
    
    def test_00_createPopoll(self,):
        """
        Create new poll in home directory & check its basic properties and behaviour
        """
        # Create poll
        content_id = "my_popoll"
        content = self.createPopoll(content_id)
        
        # Basic checks
        self.assertEqual(content.title, '')
        self.assertEqual(content.id, content_id)
        # Youpie
        Log(LOG_NOTICE, "PlonePopoll instanciated correctly !")

    def test_01_setPopoll(self,):
        """
        Set poll options
        """
        # Create poll and test that initial options are set correctly
        popoll = self.createPopoll()
        self.failUnless(popoll.getPollQuestion() == "", )
        self.failUnless(popoll.listPollChoices() == [], )
        
        # Set popoll options
        popoll.setPollOptions(
            question = "QUESTION",
            choices = [
            "A",
            "B",
            "C",
            "D",
            ],
            )

        # Ensure that they are applied
        self.failUnless(popoll.getPollQuestion() == "QUESTION")
        self.failUnless(popoll.listPollChoices() == ["A", "B", "C", "D", ])

        # Reset popoll options
        popoll.setPollOptions(
            question = "QUESTION2",
            choices = [
            "A2",
            "B2",
            "C2",
            "D2",
            ],
            )

        # Ensure that they are applied
        self.failUnless(popoll.getPollQuestion() == "QUESTION2")
        self.failUnless(popoll.listPollChoices() == ["A2", "B2", "C2", "D2", ], 
                        "Choices are invalid (%s)" % (popoll.listPollChoices()))

    def test_02_voting(self,):
        """
        Vote for an option
        """
        # Set popoll options
        popoll = self.createPopoll()
        popoll.setPollOptions(
                    question = "QUESTION",
                    choices = ["A","B","C","D",],)
    
        # (enable and) Voooote !
        popoll.enable()
        popoll.vote(index = 0)
    
        # Check results.
        # getResults returns a LIST of DICTS with the following keys :
        #        index, label, votes, percentage
        self.failUnless(popoll.getVotesCount() == 1, 
                        "Invalid number of votes taken into consideration (%d instead of 1)" % (popoll.getVotesCount()))
        dict = popoll.getResults()[0]
        ## self.failUnless(dict["index"] == 0, "Invalid option index when returning results")
        ## self.failUnless(dict["label"] == "A", "Invalid option label when returning results")
        ## self.failUnless(dict["votes"] == 1, "Invalid number of votes when returning results")
        ## self.failUnless(dict["percentage"] == 1, "Invalid percentage when returning results")


    def test_03_mass_voting(self,):
        """
        Test mass voting
        """
        # set first popoll
        poll1 = self.createPopoll("poll1")
        poll1.setPollOptions(
            question = "Uh ?",
            choices = ["1", "2", "3", "Nous irons aux bois", ],
            check_multi = 0,                                            # Disable multi-vote guardian
            )
        poll2 = self.createPopoll("poll2")
        poll2.setPollOptions(
            question = "Bah alors ?",
            choices = ["4", "5", "6", "Cueillir des cerises", ],
            check_multi = 0,                                            # Disable multi-vote guardian
            )
        poll1.enable()
        poll2.enable()

        # Vote 
        poll1.vote(0)
        poll1.vote(1)
        poll1.vote(2)
        poll1.vote(3)
        poll2.vote(3)
        poll2.vote(1)
        poll2.vote(0)
        poll2.vote(3)
        poll2.vote(1)

        # Compute tests results
        self.failUnless(poll1.getVotesCount() == 4, "Invalid number of votes taken into consideration (%d instead of 4)" % (poll1.getVotesCount()))
        self.failUnless(poll2.getVotesCount() == 5, "Invalid number of votes taken into consideration (%d instead of 5)" % (poll2.getVotesCount()))
        self.failUnless(poll1.getResults()[0][2] == 25, "Results percentage is %f instead of 25" % (poll1.getResults()[0][2], ))

    
        
        
    def test_04_vote_enabling(self,):
        """
        Test if it is possible to vote when it is enabled or not
        """
        poll1 = self.createPopoll("poll1")
        poll1.setPollOptions(
            question = "Bah alors ?",
            choices = ["4", "5", "6", ]
            )
        try:
            poll1.vote(2)
        except:
            pass
        else:
            self.fail("It should not be possible to vote when a poll is disabled")

        poll1.enable()
        poll1.vote(2)

        poll1.disable()
        try:
            poll1.vote(2)
        except:
            pass
        else:
            self.fail("It should not be possible to vote when a poll is disabled")

    def test_05_single_choice_vote(self,):
        # Non-multi-vote behaviour
        question = 'une question ?'
        choices = ['answer1', 'answer2', 'answer3']
        poll2 = self.createPopoll("poll2")
        
        self.changePollOptions(
                        poll=poll2,
                        question = question,
                        choices = choices,
                        check_multi = 0,
                        choices_number = 1,
        )
        poll2.enable()
        self.login("user2")
        #TO CHANGE
        # it is the interface that is testing how much vote are to keep, has to changed
        user_choice = 0
        if poll2.hasVoted():
            poll2.removeUserVote()
        poll2.vote(int(user_choice),)
        Log(LOG_NOTICE, poll2.hasVoted())
        
        #one person has voted
        self.assertEqual(poll2.getPersonVoteCount(), 1)
        #there is only one vote
        self.assertEqual(poll2.getVotesCount(), 1)
        
        #finally the last vote should be saved, it is for choice 0
        choice = poll2.getResults()[0]
        choice_id = choice[0];
        choice_count = choice[1];
        choice_percentage = '%0.2f'%choice[2];
        #verify vote (first choice)
        self.assertEqual(choice_count, 1)
        self.assertEqual(choice_percentage, '100.00')
        
        choice = poll2.getResults()[1]
        choice_id = choice[0];
        choice_count = choice[1];
        choice_percentage = '%0.2f'%choice[2];
        #verify vote (second choice)
        self.assertEqual(choice_count, 0)
        self.assertEqual(choice_percentage, '0.00')
    
    def test_05_multi_choice_vote(self,):
        """
        Test if a user cannot vote several times when multi-vote is checked against
        """
        
        
        # Multi-vote behaviour (default behaviour)
        poll1 = self.createPopoll("poll1")
        
        
        Log(LOG_NOTICE, self.membership.getAuthenticatedMember())
        question = "Bah alors ?"
        choices = ["4", "5", "6", ]
        #self.editPopoll(poll=poll1, _title="titre", _question=question, _answers=choices, _poll_status='enabled', _number=1, _results_visibility='visible')
        
        self.changePollOptions(
                        poll=poll1,
                        question = question,
                        choices = choices,
                        check_multi = 1,
                        choices_number = 2,
        )
        poll1.enable()
        
        self.login("user2")
        poll1.vote(0)
        poll1.vote(1)
        
        #one person has voted
        self.assertEqual(poll1.getPersonVoteCount(), 1)
        
        #total vote must be 2
        self.assertEqual(poll1.getVotesCount(), 2)
        
        #Log(LOG_NOTICE, "Mutiple choice vote !")
        
        choice = poll1.getResults()[0]
        choice_id = choice[0];
        choice_count = choice[1];
        choice_percentage = '%0.2f'%choice[2];
        #verify vote (first choice)
        self.assertEqual(choice_count, 1)
        self.assertEqual(choice_percentage, '50.00')
        
        choice = poll1.getResults()[1]
        choice_id = choice[0];
        choice_count = choice[1];
        choice_percentage = '%0.2f'%choice[2];
        #verify vote (second choice)
        self.assertEqual(choice_count, 1)
        self.assertEqual(choice_percentage, '50.00')
        
    
    def test_06_folder_poll(self,):
        """
            test if we can get the published poll in local folder appears first
        """
        poll1 = self.createPopoll("poll1")
        
    def test_07_clone_poll(self,):
        """
            BUG of copy a poll
            when a poll is copied the uiid has to be different for the copied version
        """
        self.login('user1')
        src = self.membership.getHomeFolder('user1')
        
        poll = self.createPopoll(id="popoll_copy", folder=src)
        dest = self.membership.getPersonalFolder('user1')
        dest.manage_pasteObjects(src.manage_copyObjects('popoll_copy'))

        # After a copy/paste, they should *both* have a copy
        self.failUnless(hasattr(aq_base(src), 'popoll_copy'))
        self.failUnless(hasattr(aq_base(dest), 'popoll_copy'))
        
        poll_copy = getattr(aq_base(dest), 'popoll_copy')
        self.failIf(poll_copy.getVoteId() == poll.getVoteId(), 'A copied poll must have a new generated uid.')
        
    """
    #won't work so, it the backend that change its storage not the popoll
    def test_06_migration(self, ):
        self.folder._importObjectFromFile('poll_v2_0x.zexp')
        poll = getattr(self.folder, 'poll_v2_0x')
        choice = poll.getResults()[0]
        choice_id = choice[0];
        choice_count = choice[1];
        
        poll.getResults()[1][1]
        # do reinstall to migrate
        self.installPlonePopoll()
        poll.getResults()[1][1]
    """

if __name__ == '__main__':
    framework()
else:
    def test_suite():
        from unittest import TestSuite, makeSuite
        suite = TestSuite()
        suite.addTest(makeSuite(TestPlonePopoll))
        return suite


